"""
辅助函数
"""

from pathlib import Path# noqa
import matplotlib.pyplot as plt# noqa
import scipy.io as sio# noqa


def save_matrix(matrix, file_name):
    # TODO: 存储 matrix 到 file_name.mat, mdict 的 key 为 "matrix"
    # raise NotImplementedError
    sio.savemat('%s.mat' % file_name, {'matrix': matrix})


def save_fig(matrix, file_name):
    # TODO: 将 matrix 画图保存到 file_name.jpg
    # raise NotImplementedError
    plt.imshow(matrix)
    plt.show()
    plt.imsave('%s.jpg' % file_name, matrix)


def make_dir(outdir):
    # TODO: 当目录 outdir 不存在时创建目录
    if not Path(outdir).exists():
        Path(outdir).mkdir(parents=True, exist_ok=True)



# TODO 正确导入函数 generate_matrix, save_matrix, save_fig
# from simplelayout.cli import get_options  # TODO: 保证不修改本行也可以正确导入
from simplelayout.cli.cli_generate import get_options  # 不用修改cli包下面的init
from simplelayout.generator.core import generate_matrix
from simplelayout.generator.utils import save_fig
from simplelayout.generator.utils import save_matrix
from simplelayout.generator.utils import make_dir


def main():
    # TODO 使用导入的函数按命令行参数生成数据，包括 mat 文件与 jpg 文件
    option = get_options()
    x = generate_matrix(
        option.board_grid, option.unit_grid, option.unit_n, option.positions)
    make_dir(option.outdir)
    file_path = option.outdir + '/' + option.file_name
    save_matrix(x, file_path)
    save_fig(x, file_path)


if __name__ == "__main__":
    main()



"""
数据生成的主要逻辑
"""


import numpy as np


def generate_matrix(
    board_grid: int, unit_grid: int, unit_n: int, positions: list
) -> np.ndarray:
    """生成指定布局矩阵

    Args:
        board_grid (int): 布局板分辨率，代表矩形区域的边长像素数
        unit_grid (int): 矩形组件分辨率
        unit_n (int): 组件数
        positions (list): 每个元素代表每个组件的位置
    Returns:
        np.ndarray: 布局矩阵
    """
    # raise NotImplementedError  # TODO: 实现布局矩阵的生成
    x = np.zeros([board_grid, board_grid])
    for i in positions:
        cols = ((i - 1) % int(board_grid / unit_grid)) * unit_grid
        rows = ((i - 1) // int(board_grid / unit_grid)) * unit_grid
        x[rows:rows+unit_grid, cols:cols+unit_grid] = 1
    return x



import argparse
import os


def get_options():
    parser = argparse.ArgumentParser()
    # TODO: 按 1-simplelayout-CLI 要求添加相应参数
    parser.add_argument("-b", "--board_grid", type=int, help="board_length")
    parser.add_argument("-u", "--unit_grid", type=int, help="unit_length")
    parser.add_argument("--unit_n", type=int, help="unit numbers")
    parser.add_argument("--positions", nargs='+', type=int)  # nargs='+'
    parser.add_argument("-o", "--outdir", type=str, default="example_dir")
    parser.add_argument("--file_name", type=str, default="example")
    args = parser.parse_args()
    if args.board_grid % args.unit_grid != 0:
        print("mod error")
        # exit()
    #  positions = args.positions[:-1].split(' ')  # --positions 1 4
    print(args.positions)
    if len(args.positions) != args.unit_n:
        print("grid numbers error")
        # exit()
    else:
        for pos in args.positions:
            if int(pos) >= int(args.board_grid/args.unit_grid) ** 2:
                print("too large, error")
                print(pos, int(args.board_grid/args.unit_grid) ** 2)
                # exit()
    if not os.path.exists(args.outdir):
        os.makedirs(args.outdir)
    return args



#!/usr/bin/env python
# coding=utf-8

from setuptools import setup, find_packages


setup(
    name="simplelayout-tingsongpku",  # pypi中的名称，pip或者easy_install安装时使用的名称
    version="1.0",
    author="tingsong",  # 描述性文字，可选
    author_email="tingsong@pku.edu.cn",  # 描述性文字，可选
    description=("This is a simplelayout package"),  # 描述性文字，可选
    license="GPLv3",  # 描述性文字，可选
    keywords="simple layout",  # 描述性文字，可选
    url="https://ssl.xxx.org/redmine/projects/RedisRun",  # 描述性文字，可选
    # packages=find_packages(),
    # 需要打包的目录列表,需包含src/simplelayout，需指定src目录？？
    package_dir={'': 'src'},
    packages=find_packages(where="src"),  # Required
    # 需要安装的依赖包，有些是自己编写的或者网上的
    install_requires=[
        'setuptools>=16.0',
        'pytest',
        'numpy',
        'matplotlib',
        'scipy'
    ],

    # 添加这个选项，在windows下Python目录的scripts下生成exe文件
    # 注意：模块与函数之间是冒号:
    # 设置了在命令行中如何使用 XX_module 中的 main 函数，直接命令行调用
    entry_points={'console_scripts': [
        'simplelayout = simplelayout.__main__:main',
    ]},

    # long_description=read('README.md'),
    classifiers=[  # 程序的所属分类列表，描述性文字，可选
        "Development Status :: 3 - Alpha",
        "Topic :: Utilities",
        "License :: OSI Approved :: GNU General Public License (GPL)",
    ],
    # 此项需要，否则卸载时报windows error
    zip_safe=False
)
